
{ 4I22 DEMO SOFTWARE }

program INT4I22;

{$R-}    {Range checking be off }
{$I-}    {No I/O checking }
{$S-}    {No stack checking}

{$M 1024,1024,4096}           { use minimum memory, but a large stack...      }
                              { don't make the heap any smaller or you'll be in heap big trouble }

uses DOS;

{$I Beepy.pas}                { include speaker beep stuff }
{$I 8259.pas}                 { include interrupt stuff }

{$I 4I22LOW }                 { include low level 4I22 I/O }

{ Set prescaler (counter 3C) to 100K Hz square wave }
{  set counter 1A ( our interrupt generater ) to 200 HZ square wave }
var

TheIntNum : byte;

procedure LocalDelay(time : word);
var iloop,oloop : word;
begin
  for  oloop := 1 to time do
  begin
    for iloop := 1 to 100 do ;
  end;
end;

procedure BeepISR;
interrupt;
begin
  EnableInterrupts;
  BeepOn(800);
  LocalDelay(30);
  BeepOff;
  if TheIntNum < 8 then IssueEOI else IssueEOI2;
end;

procedure InitCounters;
begin
  SetCounterMode(Counter3C,SquareWave);  { Used as prescaler }
  LoadCounter(Counter3C,100);            { 100K Hz = 10M Hz / 100 }
  SetCounterMode(Counter1A,SquareWave);
  LoadCounter(Counter1A,33333);          { 3 Hz = 100K Hz / 33333 }
end;

procedure Setup4I22IntHardware(intnum : byte);
begin
  { setup the 4I22 card interrupt logic }
  SetIntMode(IntModeOff);
  { make sure interrupt driver is off before changing ints }

  SetIntNum(intnum);           { select the interrupt that the 4I22 drives }
  MaskIntB;                    { make sure Timer 1B can't generate ints }
  UnMaskIntA;                  { and make sure timer 1A can... }
  SetIntMode(IntModeOn);       { finally, enable 4I22 int driver }
end;

procedure Install;
begin
  writeln;
  writeln('4I22INT --- Installing annoying beep background task...');
  DisableInterrupts;
  SetHardIntVec(TheIntNum,@BeepISR);
  InitCounters;
  Setup4I22IntHardWare(TheIntNum);             { get 4I22 to drive the requested IRQ line }
  UnmaskInterrupt(TheIntNum);                  { on the 8259 }
  EnableInterrupts;
  keep(0);                                     { terminate and stay resident }
end;

function GetIntNum : boolean;
var
retcode : integer;
begin
  GetIntNum := false;
  val(ParamStr(1),TheIntNum,Retcode);
  if retcode = 0 then
  { do some really rudimentary validity checking }
  begin
    if TheIntNum < 16 then
    begin
      if TheIntNum > 1 then
      begin
        GetIntNum := true;
      end;
    end;
  end;
end;

procedure Usage;
begin
  writeln;
  writeln('USAGE   4I22INT   IRQ');
  writeln('Where IRQ is a valid and free interrupt line ');
  writeln;
  writeln('4I22INT Installs background task that generates');
  writeln('annoying beep at 3 beeps per sec...');
  writeln;
end;

begin  { main }
  if GetIntNum then Install else usage;
end.
