
{ 4I22 low level procedures }

const
{ First define the 4I22 port locations }

{ 8254 counters }
Counter1A    = Cardbase + $0;
Counter1B    = Cardbase + $1;
Counter1C    = Cardbase + $2;
Counter1Cont = Cardbase + $3;         { Normally not used - generated }
Counter2A    = Cardbase + $4;
Counter2B    = Cardbase + $5;
Counter2C    = Cardbase + $6;
Counter2Cont = Cardbase + $7;         { Normally not used - generated }
Counter3A    = Cardbase + $8;
Counter3B    = Cardbase + $9;
Counter3C    = Cardbase + $A;
Counter3Cont = Cardbase + $B;         { Normally not used - generated }

{ parallel port stuff }

PPIA         = Cardbase + $C;
PPIB         = Cardbase + $D;
PPIC         = Cardbase + $E;
PPICont      = Cardbase + $F;

{ Addressable latch bits }

ISel0        = CardBase + $8000;
ISel1        = CardBase + $8001;
ISel2        = CardBase + $8002;
ISel3        = CardBase + $8003;
IMode0       = CardBase + $8004;
IMode1       = CardBase + $8005;
IMaskA       = CardBase + $8006;
IMaskB       = CardBase + $8007;

{ some aglomerations of addressable latch bits }

IntModeOff   = 0;
IntModeOn    = 1;
IntModeSh    = 2;
IntModeShPD  = 3;

{ Then define the counter mode control bytes }

PeriodicInt    : byte = $0;
HardwOneShot   : byte = $1;
RateGenerator  : byte = $2;
SquareWave     : byte = $3;
SoftwStrobe    : byte = $4;
HardwStrobe    : byte = $5;

{ Some useful counter masks and command bytes }

ReadBackCommand  : byte = $EE;
OutBitMask     : byte = $80;
NullCountMask  : byte = $40;

{ And a few useful PPI mode bytes }

AllOut         : byte = $80;
Allin          : byte = $9B;
OutABInC       : byte = $83;

procedure SetCounterMode(counterPort : word; counterMode : byte);
var modeByte : byte;
var controlPort : word;

begin
  modeByte := lo(counterPort shl 6);          { generate counter select bits }
  modeByte := modeByte or counterMode shl 1;  { or in mode bits }
  modeByte := modeByte or $30;                { set RW bits for least - most, binary }
  controlPort := counterPort or $3;           { generate control port address }
  port[controlPort] := modeByte ;             { write mode byte }
end; { SetCounterMode }

procedure LoadCounter(counterPort : word; count : word);
var loByte : byte ;
var hiByte : byte ;

begin
  loByte := lo(count);                        { Least significant byte }
  hibyte := hi(count);                        { Most significant byte }
  port[counterPort] := loByte;                { Write least }
  port[counterPort] := hiByte;                { Write most }
end; { LoadCounter }

procedure ReadCounter(counterPort : word; var count : word);
var loByte : word ;
var hiByte : word ;

begin
  loByte := port[counterPort];                { Read least significant byte }
  hiByte := port[counterPort];                { Read most significant byte }
  count  := lobyte and $00FF;                 { Build word }
  count  := swap(count);
  count  := count or (hiByte and $00FF);
  count  := swap(count);
end; { LoadCounter }

procedure LatchCounter(counterPort : word);
var modebyte : byte;
var controlPort : word ;

begin
  modeByte := lo(counterPort shl 6);          { generate counter select bits with RW bits =0 }
  controlPort := counterPort or $3;           { generate control port address }
  port[controlPort] := modeByte ;             { write latch mode word }
end; { LatchCounter }

procedure ReadCounterStatus(counterPort : word; var status : byte);
var controlPort : word ;
var counterSelect : byte ;
var counterSelectBits : byte ;
begin
  controlPort := counterPort or $3;           { Generate control port address }
  counterSelect := (counterPort and 3);
  counterSelectBits := 2;
  counterSelectBits := counterSelectBits shl counterSelect;
  port[controlPort] := ReadBackCommand or counterSelectBits;       { latch status command }
  status := port[counterPort];                { Read status byte }
end; { ReadCounterStatus }

procedure SetPPIMode(PPImode : byte);

begin
  port[PPICont] := PPIMode;
end; { SetPPIMode }

procedure ReadPPI(PPIPort : word;  var PPIData : byte);

begin
  PPIdata := port[PPIPort];
end; { ReadPPI }

procedure WritePPI(PPIPort : word; PPIdata : byte);

begin
  port[PPIPort] := PPIData;
end; { Write PPIData }

procedure SetIntMode(imode : byte);
begin
  port[IMode0] := imode;
  port[IMode1] := (imode shr 1);
end;

procedure SetIntNum(IntNum : byte);
begin
  port[ISel0] := intnum;
  port[ISel1] := intnum shr 1;
  port[ISel2] := intnum shr 2;
  port[ISel3] := intnum shr 3;
end;

procedure MaskIntA;
begin
  port[IMaskA] := 0;
end;

procedure MaskIntB;
begin
  port[IMaskB] := 0;
end;

procedure UnMaskIntA;
begin
  port[IMaskA] := 1;
end;

procedure UnMaskIntB;
begin
  port[IMaskB] := 1;
end;

